% =========================================================================
%           Group Sparse CR-RLS Adaptive Algorithms Realization
% =========================================================================
%
% Realizes the Group Sparse RLS algorithm curves shown as in Figs.1, 2 of the following paper:
% Paper:  "Group Sparse RLS Algorithms", 
% International Journal of Adaptive Control and Signal Processing, by E.M. Eksioglu 
% accepted for publication, 2014.
% Code written by Ender M. Eksioglu 
% January 2014
% =========================================================================
clear all;
close all;
clc
for data_length=1:1
%
NR=100; % number of iterations
gm_factor=0;
norm_error=1;
%%
num_elements=1;
num_coefs=data_length*64; %%L
%%
SNRR=1; % 10 dB
SNRRvals % assigns the necessary parameters, noise variance etc.
%%
block_size=4; %%S
block_sparsity=1;
block_occupy=4;
%%
S=block_occupy*block_sparsity;
L=num_coefs;
block_num=L/block_size;
%%
epss1=0.01;  % l1 RRLS
epss2=0.00001;  % l21 RRLS 
epss3=0.00001;  % l20 RLS
%%
arfilter=[1];
%%
numepoch=20;
N=numepoch*100+L-1;

ff=0.995;
rf=1;
%% SNR estimate
SNRdBest=0;
%%
MSD1=zeros(N,1);

MSD2=zeros(N,1);

MSD3=zeros(N,1);

MSD4=zeros(N,1);

MSD5=zeros(N,1);

MSD6=zeros(N,1);

MSD7=zeros(N,1);

MSDp=zeros(N,1);

for Nr=1:NR
    Nr
%% Create block sparse data
    h=sqrt(1/S)*BlockSparseDataGenerator(num_elements, num_coefs, block_size,  block_sparsity, block_occupy);
%%   Group ordering
d_orig=repmat([1:block_num],block_size,1);
d_orig=d_orig(:).';
%%
ell_2_1_norm_original=0;
ell_2_0_norm_approx_original=0;
ell_infty_1_norm_original=0;
for jjj = 1:block_num
    groups{jjj} = find(d_orig == jjj);
    coeffs_dummy=h(groups{jjj});
    ell_2_1_norm_original=ell_2_1_norm_original+norm(coeffs_dummy,2);
    ell_infty_1_norm_original=ell_infty_1_norm_original+max(abs(coeffs_dummy));
end

%1) l1 RLS
ell_1_norm_original=norm_error*norm(h,1);
norm1=ell_1_norm_original;
%2) l0 RLS
ell_0_norm_original=norm_error*S;
norm2=ell_0_norm_original;
% 3) l10 RLS
ell_1_0_norm_original=norm_error*block_sparsity;
norm3=ell_1_0_norm_original;
% 4) l20 RLS
ell_2_0_norm_original=norm_error*block_sparsity;
norm4=ell_2_0_norm_original;
% 5) l_(infty,1)
ell_infty_1_norm_original=norm_error*ell_infty_1_norm_original;
norm5=ell_infty_1_norm_original;
% 6) l 21
ell_2_1_norm_original=norm_error*ell_2_1_norm_original;
norm6=ell_2_1_norm_original;
%pause
%%
    x=randn(N,1);
    x=filter(1,arfilter,x);
    y=conv(x,h);
    Spower=norm(y,2)^2/length(y);
    
    SNRdBest=SNRdBest+10*log10(Spower/noise_variance);
    % noisy data
    y=y+sqrt(noise_variance)*randn(length(y),1);
           
    W1=zeros(L,1);
    
    W2=zeros(L,1);
    
    W3=zeros(L,1);
    
    W4=zeros(L,1);
    
    W5=zeros(L,1);
    
    W6=zeros(L,1);
    
    W7=zeros(L,1);
    
    Wp=zeros(L,1);   
    %% 
    K=[];

    P=1/rf*eye(L,L);

%% time iteration
    for n=L:N

  b=b_z+b_dif*n/N;
  beta1=beta_z1+beta_dif1*n/N;
  beta2=beta_z2+beta_dif2*n/N;
 
        X=x(n:-1:n-L+1);
                     
        y1(n)=W1'*X;
        
        y2(n)=W2'*X;
        
        y3(n)=W3'*X;
        
        y4(n)=W4'*X;
        
        y5(n)=W5'*X;
        
        y6(n)=W6'*X;
        
        y7(n)=W7'*X;
        
        yp(n)=Wp'*X;
        
       
        e1(n)=y(n)-y1(n);
        
        e2(n)=y(n)-y2(n);
        
        e3(n)=y(n)-y3(n);
        
        e4(n)=y(n)-y4(n);
        
        e5(n)=y(n)-y5(n);
        
        e6(n)=y(n)-y6(n);
        
        e7(n)=y(n)-y7(n);
        
        ep(n)=y(n)-yp(n);
                    
        K=P*X/(ff+X'*P*X);                  
%% calculate group of subgradients for group sparsity, mixed norm
%% l1 RLS or RRLS nongroup
subgvec1=sign(W1);
ell_1_norm=norm(W1,1);
normdiff1=ell_1_norm-norm1;
%% l0 RLS NONGROUP
  subgvec2=b.*(sign(W2).*(1-b.*abs(W2)).*(b*abs(W2)<1));
  ell_2_norm=sum(abs(W2)>0.0001);
  normdiff2=ell_2_norm-norm2;
%%
subgvec3=[];
subgvec4=[];
subgvec5=[];
subgvec6=[];
ell_3_0_norm=0;
ell_4_0_norm=0;
ell_5_0_norm=0;
ell_6_0_norm=0;

for jjj = 1:block_num
  %% lp0 exponential Group
    %% L10 
    coeffs_dummy=W3(groups{jjj});
    subg3=beta1*sign(coeffs_dummy).*exp(-beta1*norm(coeffs_dummy,1)); 
    ell_3_0_norm=ell_3_0_norm+(norm(coeffs_dummy,2)>0.0001);
    subgvec3=[subgvec3; subg3];
    %% L20 
    coeffs_dummy=W4(groups{jjj});
    subg4=beta2*coeffs_dummy/(norm(coeffs_dummy,2)+epss3)*(1-beta2*norm(coeffs_dummy,2))*(beta2*norm(coeffs_dummy,2)< 1);
    ell_4_0_norm=ell_4_0_norm+(norm(coeffs_dummy,2)>0.0001);
    subgvec4=[subgvec4; subg4];
    %% l_(infty,1)  
     coeffs_dummy=W5(groups{jjj});
     [Yy,Ii] = max(abs(coeffs_dummy));
     subg5=zeros(size(coeffs_dummy));
     subg5(Ii)=sign(coeffs_dummy(Ii));
     ell_5_0_norm=ell_5_0_norm+abs(Yy);
      subgvec5=[subgvec5; subg5];
    %% l21  
    coeffs_dummy=W6(groups{jjj});
    subg6=coeffs_dummy/(norm(coeffs_dummy,2)+epss2);
    ell_6_0_norm=ell_6_0_norm+norm(coeffs_dummy,2);   
    subgvec6=[subgvec6; subg6];
end

 normdiff3=ell_3_0_norm-norm3;
 normdiff4=ell_4_0_norm-norm4;
 normdiff5=ell_5_0_norm-norm5;
 normdiff6=ell_6_0_norm-norm6;

%%
       alpha=trace(P)/L;
        
        %% l1 RLS or RRLS  no group
        gm1(n)=max(0,2*(alpha* normdiff1 + gm_factor*subgvec1'*P*(Wp-W1)) / norm(P*subgvec1,2).^2 );
        %% l0 exponential no group
        gm2(n)=max(0,2*(alpha* normdiff2 + gm_factor*subgvec2'*P*(Wp-W2))  / norm(P*subgvec2,2).^2  ); 
        %% l0 exponential group surrogate l10
        gm3(n)=max(0,2*(alpha*normdiff3 + gm_factor*subgvec3'*P*(Wp-W3)) / norm(P*subgvec3,2).^2 );
        %% l0 exponential group surrogate l20
        gm4(n)=max(0,2*(alpha*normdiff4 + gm_factor*subgvec4'*P*(Wp-W4)) / norm(P*subgvec4,2).^2 );      
        %% l_(infty,1)
        gm5(n)=max(0,2*(alpha*normdiff5 + gm_factor*subgvec5'*P*(Wp-W5))/ norm(P*subgvec5,2)^2 );    
        %% l21 group 
        gm6(n)=max(0,2*(alpha*normdiff6 + gm_factor*subgvec6'*P*(Wp-W6))/ norm(P*subgvec6,2)^2 );      
        %%
        P=(1/ff*P)-(1/ff*K*X'*P);  
        P=(P+P')/2;
        %% L1-RLS (RRLS)
         W1=W1+K*conj(e1(n))+gm1(n)*(ff-1)*P*subgvec1;
        %% l0 exponential nongroup
         W2=W2+K*conj(e2(n))+gm2(n)*(ff-1)*P*subgvec2;
        %% l0 exponential group surrogate l10
        W3=W3+K*conj(e3(n))+gm3(n)*(ff-1)*P*subgvec3;
        %% l0 exponential group surrogate l20
        W4=W4+K*conj(e4(n))+gm4(n)*(ff-1)*P*subgvec4;
         %% l_(infty,1)
         W5=W5+K*conj(e5(n))+gm5(n)*(ff-1)*P*subgvec5;
        %% l21
        W6=W6+K*conj(e6(n))+gm6(n)*(ff-1)*P*subgvec6;  
        %% standard RLS  
        Wp=Wp+K*conj(ep(n));
     %%        
        msdp(n)=(h-Wp)'*(h-Wp);
        
        msd1(n)=(h-W1)'*(h-W1);
        
        msd2(n)=(h-W2)'*(h-W2);
        
        msd3(n)=(h-W3)'*(h-W3);
         
        msd4(n)=(h-W4)'*(h-W4);
        
        msd5(n)=(h-W5)'*(h-W5);
        
        msd6(n)=(h-W6)'*(h-W6);
        
    end

    MSD1=MSD1+transpose(msd1);
    
    MSD2=MSD2+transpose(msd2);
    
    MSD3=MSD3+transpose(msd3);
    
    MSD4=MSD4+transpose(msd4);
    
    MSD5=MSD5+transpose(msd5);
     
    MSD6=MSD6+transpose(msd6);
    
    MSDp=MSDp+transpose(msdp);

end
SNRdB=SNRdBest/NR
save results MSD1 MSD2 MSD3 MSD4 MSD5 MSD6 MSDp 
%%
figure

%set(gca,'box','off');
set(gca,'YScale','log');
set(gca,'box','on');
if  SNRR==1
ylim([10^-3 1.5]);
elseif SNRR==2
  ylim([10^-4 1.5]);
    end      
hold on
colors =  {'k';  'k';  'k';'k';'k';'k';'k'};
lines =   {'-'; '-'; '-'; '-'; '-'; '-'; '-'};
markers = {'x';  'v';  'd';  '*'; 'o'; '+'; 's'};
%
for k = [7 1 2 3 4 5 6]
    plot(-1,0,[colors{k},lines{k},markers{k}],'markersize',10,'linewidth',1);
    %legend({'l_1-RLS'})
end
legend({'RLS','l_1-RLS','l_0-RLS','l_{1,0}-GRLS','l_{2,0}-GRLS', 'l_{\infty,1}-GRLS','l_{2,1}-GRLS'})

semilogy(1:100:numepoch*100,MSDp(L:100:N)/NR,[colors{7},markers{7}],'markersize',8,'linewidth',1);
semilogy(1:100:numepoch*100,MSD1(L:100:N)/NR,[colors{1},markers{1}],'markersize',8,'linewidth',1);
semilogy(1:100:numepoch*100,MSD2(L:100:N)/NR,[colors{2},markers{2}],'markersize',8,'linewidth',1);
semilogy(1:100:numepoch*100,MSD3(L:100:N)/NR,[colors{3},markers{3}],'markersize',8,'linewidth',1);
semilogy(1:100:numepoch*100,MSD4(L:100:N)/NR,[colors{4},markers{4}],'markersize',8,'linewidth',1);
semilogy(1:100:numepoch*100,MSD5(L:100:N)/NR,[colors{5},markers{5}],'markersize',8,'linewidth',1);
semilogy(1:100:numepoch*100,MSD6(L:100:N)/NR,[colors{6},markers{6}],'markersize',8,'linewidth',1);

semilogy(MSDp(L:N)/NR,[colors{7},lines{7}]); % standard 
semilogy(MSD1(L:N)/NR,[colors{1},lines{1}]);  % l1 RLS or RRLS non group - no group
semilogy(MSD2(L:N)/NR,[colors{2},lines{2}]);  % l0 exponential nongroup
semilogy(MSD3(L:N)/NR,[colors{3},lines{3}]); % l0 exponential group surrogate l10
semilogy(MSD4(L:N)/NR,[colors{4},lines{4}]); % l0 exponential group surrogate l20
semilogy(MSD5(L:N)/NR,[colors{5},lines{5}]); % l infty_1 reweighted group
semilogy(MSD6(L:N)/NR,[colors{6},lines{6}]); % l21 reweighted group


set(gca,'FontSize',11);
%set(gca,'linewidth',1.4);
xlabel('Iterations','FontSize',12);  
ylabel('MSD','FontSize',12);  
saveas(gcf, 'fig_Group_Sparse_RLS', 'fig')

finalval{data_length}(1)=MSD1(N)/NR;
finalval{data_length}(2)=MSD2(N)/NR;
finalval{data_length}(3)=MSD3(N)/NR;
finalval{data_length}(4)=MSD4(N)/NR;
finalval{data_length}(5)=MSD5(N)/NR;
finalval{data_length}(6)=MSD6(N)/NR;
finalval{data_length}(7)=MSDp(N)/NR;
save finalvals finalval
end